@@ -22,9 +22,12 @@ import base64
 import  copy
 import pickle
 from . import __version__
+from . import custom_formatters
 from itertools import cycle
 import re
 
+import logging
+
 from .data_products import NumpyDataProduct,BinaryData,ApiCatalog
 
 __all__=['Request','NoTraceBackWithLineNumber','NoTraceBackWithLineNumber','RemoteException','DispatcherAPI']
@@ -59,17 +62,33 @@ class RemoteException(NoTraceBackWithLineNumber):
 def safe_run(func):
 
     def func_wrapper(*args, **kwargs):
-        try:
-            return func(*args, **kwargs)
-        except Exception as e:
-           message =  'the remote server response is not valid\n'
-           message += 'possible causes: \n'
-           message += '- connection error\n'
-           message += '- wrong credentials\n'
-           message += '- error on the remote server\n'
-           message += '\n exception message: '
-           message += '%s'%e
-           raise RemoteException(message=message)
+        logger = logging.getLogger(func.__name__)
+
+        self = args[0] # because it really is
+
+        n_tries_left = self.n_max_tries
+        while True:
+            try:
+                return func(*args, **kwargs)
+            except Exception as e:
+                message = ''
+                message += '\nunable to complete API call'
+                message += '\nin ' + str(func) + ' called with:'
+                message += '\n... ' + ", ".join([str(arg) for arg in args])
+                message += '\n... ' + ", ".join([k+": "+str(v) for k,v in kwargs])
+                message += '\npossible causes:'
+                message += '\n- connection error'
+                message += '\n- wrong credentials'
+                message += '\n- error on the remote server'
+                message += '\n exception message: '
+                message += '\n\n%s\n'%e
+
+                n_tries_left -= 1 
+
+                if n_tries_left > 0:
+                    logger.warning("problem in API call, %i tries left:\n%s\n sleeping %i seconds until retry", n_tries_left, message, self.retry_sleep_s)
+                else:
+                    raise RemoteException(message=message)
 
     return func_wrapper
 
@@ -81,6 +100,11 @@ class DispatcherAPI(object):
         self.cookies=cookies
         self.set_instr(instrument)
 
+        self.n_max_tries = 20
+        self.retry_sleep_s = 5
+
+        self.custom_progress_formatter = custom_formatters.find_custom_formatter(instrument)
+
         if self.host.startswith('htt'):
             self.url=host
         else:
@@ -96,6 +120,9 @@ class DispatcherAPI(object):
 
         self._progress_iter = cycle(['|', '/', '-', '\\'])
 
+    def set_custom_progress_formatter(self, F):
+        self.custom_progress_formatter = F
+
     @classmethod
     def build_from_envs(cls):
         cookies_path = os.environ.get('ODA_API_TOKEN')
@@ -111,11 +138,18 @@ class DispatcherAPI(object):
     def set_instr(self,instrument):
         self.instrument=instrument
 
-
-
-    def _progess_bar(self,info=''):
+    def _progress_bar(self,info=''):
         print("\r %s the job is working remotely, please wait %s"%(next(self._progress_iter),info),end='')
 
+    def format_custom_progress(self, full_report_dict_list):
+        F = getattr(self, 'custom_progress_formatter', None)
+
+        if F is not None:
+            return F(full_report_dict_list)
+
+        return ""
+
+
     @safe_run
     def request(self,parameters_dict,handle='run_analysis',url=None):
         if 'scw_list' in parameters_dict.keys():
@@ -137,26 +171,37 @@ class DispatcherAPI(object):
         if query_status != 'done' and query_status != 'failed':
             print ('the job has been submitted on the remote server')
 
+        t0 = time.time()
+
         while query_status != 'done' and query_status != 'failed':
             parameters_dict['query_status']=query_status
             parameters_dict['job_id'] = job_id
-            #print('-> sent3')
             res = requests.get("%s/%s" % (url,handle), params=parameters_dict,cookies=self.cookies)
-            query_status =res.json()['query_status']
-            job_id = res.json()['job_monitor']['job_id']
-            info='status=%s - job_id=%s '%(query_status,job_id)
-            self._progess_bar(info=info)
-            #print('-> sent4')
+            res_json = res.json()
+            query_status =res_json['query_status']
+            job_id = res_json['job_monitor']['job_id']
+            info = 'status=%s job_id=%s in %d messages since %d seconds'%(
+                        query_status, 
+                        job_id, 
+                        len(res_json['job_monitor']['full_report_dict_list']),
+                        time.time() - t0,
+                    )
+
+            custom_info = self.format_custom_progress(res_json['job_monitor']['full_report_dict_list'])
+            if custom_info != "":
+                info += "; " + custom_info
+
+
+            self._progress_bar(info=info)
 
             time.sleep(2)
 
         print("\r", end="")
         print('')
         print('')
-        if  res.json()['exit_status']['status']!=0:
+        if res.json()['exit_status']['status']!=0:
             self.failure_report(res)
 
-
         #print('job_monitor', res.json()['job_monitor'])
         #print('query_status', res.json()['query_status'])
         #print('products', res.json()['products'].keys())
@@ -355,7 +400,8 @@ class DispatcherAPI(object):
                 data.append(ApiCatalog(js['products']['catalog'],name='dispatcher_catalog'))
 
             if 'astropy_table_product_ascii_list' in res.json()['products'].keys():
-                data.extend([ascii.read(table_text) for table_text in js['products']['astropy_table_product_ascii_list']])
+                #print()
+                data.extend([ascii.read(table_text['ascii']) for table_text in js['products']['astropy_table_product_ascii_list']])
 
             if 'astropy_table_product_binary_list' in res.json()['products'].keys():
                 #for  table_binary in js['products']['astropy_table_product_binary_list']:
@@ -367,7 +413,9 @@ class DispatcherAPI(object):
                 data.extend([ascii.read(table_binary) for table_binary in js['products']['astropy_table_product_binary_list']])
 
             d=DataCollection(data,instrument=instrument,product=product)
-
+            for p in d._p_list:
+                if hasattr(p,'meta_data') is False and hasattr(p,'meta') is True:
+                    p.meta_data = p.meta
         else:
             self._decode_res_json(res.json()['products']['instrumet_parameters'])
             d=None
